package LATMOS::Accounts::Web::Controller::Admin::Requests;
use Moose;
use namespace::autoclean;
use LATMOS::Accounts::Log;

BEGIN {extends 'Catalyst::Controller'; }

=head1 NAME

LATMOS::Accounts::Web::Controller::Admin::Requests - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;

}

sub default :LocalPath {
    my ( $self, $c, undef, undef, $id, $action) = @_;

    my $base = $c->model('Accounts')->db;

    $c->stash->{request} = $base->get_datarequest($id);

    $c->stash->{form} = $c->model(
        'AttrForms',
        '',
        $c->stash->{request},
        $c->model('Accounts')->db
    );

    if ($action) {
        $c->go('create');
    } elsif ($c->req->param('_validate')) {
        if ($c->stash->{request}->is_for_new_object) {
            my %attrs = ();
            foreach ($c->stash->{request}->attributes) {
                my @values =
                $c->req->param($c->stash->{form}->attr_field_name($_));
                $attrs{$_} = @values > 1 ? [@values] : $values[0];
            }
            $c->session->{reqinfo} = {};
            my %newvalues = $c->stash->{request}->_prepare_attrs(%attrs);
            foreach my $attr (keys %newvalues) {
                $c->session->{reqinfo}{contents}{$attr} = $newvalues{$attr};
            }
            $c->go('create');
        } else {
            $c->go('update');
        }
    } elsif ($c->req->param('_cancel')) {
        $c->stash->{request}->register_discard($c->req->param('_comment'));
        $c->stash->{request}->base->commit;
        $c->res->redirect($c->uri_for('/admin', 'requests'));
    }
}

sub create : Private {
    my ( $self, $c ) = @_;

    my $ochelper = $c->model('Accounts')->db->ochelper($c->stash->{request}->otype);
    my $info = $c->session->{reqinfo};
    my $form = $c->stash->{form};
    foreach my $attr (@{$info->{ask} || []}) {
        $info->{contents}{$attr} = $c->req->param($form->attr_field_name($attr));
    }
    if ($info->{name}{ask}) {
        $info->{name}{content} = $c->req->param('_name');
    }

    my $status;
    ($status, $info) = $ochelper->step($info);
    $c->session->{reqinfo} = $info;

    if ($status eq 'CREATED') {
        $c->stash->{request}->_register_applied($c->req->param('_comment'));
        $c->stash->{request}->base->commit;
        $c->log->info(
            sprintf(
                "Object %s %s created\n",
                $c->stash->{request}->otype,
                $info->{name}{content},
            )
        );
        $c->res->redirect(
            $c->uri_for(
                '/' . $c->stash->{request}->otype . 's',
               $info->{name}{content},
           )
        ); 
    } elsif ($status eq 'ERROR') {
        $c->go('error');
    } elsif ($status eq 'NEEDINFO') {
        foreach my $attr (@{$info->{ask} || []}) {
            $c->req->params->{$form->attr_field_name($attr)} = $info->{contents}{$attr};
        }
    }
}

sub update : Private {
    my ( $self, $c ) = @_;

    my %attrs;
    foreach ($c->stash->{request}->attributes) {
        my @values =
            $c->req->param($c->stash->{form}->attr_field_name($_));
        $attrs{$_} = @values > 1 ? [@values] : $values[0];
    }
    my $comment = $c->req->param('_comment');
    if (!$c->stash->{request}->apply_to_object($comment, %attrs)) {
        $c->go('error');
    } else {
        $c->stash->{request}->base->commit;
    }
}

sub error : Private {
    my ( $self, $c ) = @_;
    
    $c->stash->{page}{error} =
        'Erreur lors de la création de l\'objet: ' .
        LATMOS::Accounts::Log::lastmessage(LA_ERR)
}

=head1 AUTHOR

olivier

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
